/***********************************************************************/
/*                                                                     */
/*                              Pio (C++)                              */
/*                                                                     */
/*                    Polyglot Input/Output Library                    */
/*                                                                     */
/*               EPI Pomdapi, INRIA Paris - Rocquencourt               */
/*                                                                     */
/*  Copyright 2005-2014 INRIA.                                         */
/*  Institut National de Recherche en Informatique et en Automatique.  */
/*  All rights reserved. This file is distributed only by permission.  */
/*                                                                     */
/*  Estelle Marchand <Estelle.Marchand@inria.fr>                       */
/*                                                                     */
/***********************************************************************/

/* $Id: typed_value.hpp,v 1.9 2014/01/10 19:22:09 fclement Exp $ */

#include "boolean.hpp"
#include "lexem.hpp"
#include "vector_type.hpp"
#include "matrix_type1.hpp"
#include "matrix_type2.hpp"

#ifndef TYPED_VALUE_H
#define TYPED_VALUE_H

namespace io{

  enum Typed_value_type{EMPTYtv,LEXEM,VECTOR,MATRIX1,MATRIX2};

  class Typed_value{
    Typed_value_type typed_value_type;
    Elem<Lexem> scalar;
    Elem<Vector_type> vect;
    Elem<Matrix_type1> matrix1;
    Elem<Matrix_type2> matrix2;
  public:
    Typed_value_type type();
    //constructeurs:
    Typed_value(){typed_value_type=EMPTYtv;}
    Typed_value(const Lexem&);
    Typed_value(const Vector_type&);
    Typed_value(const Matrix_type1&);
    Typed_value(const Matrix_type2&);
    template< class dataType > Typed_value(int m,int n,dataType* &M,char c='C'){matrix1.Some();matrix1.data()=Matrix_type1(m,n,M,c);typed_value_type=MATRIX1;}
    template< class dataType > Typed_value(int m,int n,dataType** &M){matrix2.Some();matrix2.data()=Matrix_type2(m,n,M);typed_value_type=MATRIX2;}
    template< class dataType > Typed_value(const vector<dataType> & v){vect.Some();vect.data() = Vector_type(v);
    typed_value_type=VECTOR;}
    template< class dataType > Typed_value(const dataType & s){scalar.Some();scalar.data() = Lexem(s);
    typed_value_type=LEXEM;}
    //convertisseurs:
    Lexem & to_Lexem();
    Matrix_type1 & to_Matrix_type1();
    Matrix_type2 & to_Matrix_type2();
    Vector_type & to_Vector_type();
    int & to_int();
    boolean & to_boolean();
    double & to_float();
    string & to_string();
    int_couple & to_int_couple();
    int_triple & to_int_triple();
    float_couple & to_float_couple();
    float_triple & to_float_triple();
    boolean_couple & to_boolean_couple();
    boolean_triple & to_boolean_triple();
    vector<int> & to_int_vector();
    vector<double> & to_float_vector();
    vector<string> & to_string_vector();
    vector<int_couple> & to_int_couple_vector();
    vector<int_triple> & to_int_triple_vector();
    vector<float_couple> & to_float_couple_vector();
    vector<float_triple> & to_float_triple_vector();
    virtual ~Typed_value();
  }; 
  

  //   class OTyped_value{
  //   public:
  //     virtual Elem<int> & value();
  //     OTyped_value(){}
  //   };
  //   class Int{
  //   public:
  //     int i;
  //     Int(int _i){i=_i;}
  //   };
  //   class Float{
  //   public:
  //     double f;
  //     Float(double _f){f=_f;}
  //   };
  //   template<class dataType> class OLexem:public OTyped_value{
  //     dataType x;
  //   public:
  //     OLexem(dataType _x){x=_x;}
  //     virtual dataType & value(){return x;}
  //   };
  //   template<class dataType> class OVector:public OTyped_value{
  //     vector<dataType> x; 
  //   public:
  //     OVector(vector<dataType> _x){x=_x;}
  //     virtual vector<dataType> & value(){return x;}
  //   };
  //   class OMatrix_type1:public OTyped_value{
  //     double* x;
  //     int _n_rows;
  //     int _n_columns;
  //   public:
  //     OMatrix_type1(int m,int n,double* X):x(X){_n_rows=m;_n_columns=n;}
  //     int & n_rows(){return _n_rows;}
  //     int & n_columns(){return _n_columns;}
  //     virtual double* & value_(){return x;}
  //   };
  //   class OMatrix_type2:public OTyped_value{
  //     double** x;
  //     int _n_rows;
  //     int _n_columns;
  //   public:
  //     OMatrix_type2(int m,int n,double** X):x(X){_n_rows=m;_n_columns=n;}
  //     int & n_rows(){return _n_rows;}
  //     int & n_columns(){return _n_columns;}
  //     virtual double** & value_(){return x;}
  //   };
}

#endif
