CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                     C
C                              Ref-indic                              C
C                                                                     C
C                        Refinement indicators                        C
C                                                                     C
C               EPI Pomdapi, INRIA Paris - Rocquencourt               C
C                                                                     C
C  Copyright (c) 2013-2014 INRIA.                                     C
C  Institut National de Recherche en Informatique et en Automatique.  C
C  All rights reserved. This file is distributed only by permission.  C
C                                                                     C
C  Hend Ben Ameur <hbenameur@yahoo.ca>                                C
C  Francois Clement <Francois.Clement@inria.fr>                       C
C  Pierre Weis <Pierre.Weis@inria.fr>                                 C
C                                                                     C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

C     $Id: main.fpp,v 1.1 2014/06/23 14:07:13 fclement Exp $

#include "worker_constants.h"
#include "model_constants.h"

      program main
C     Main program.
      
      implicit none
      
C     External routines.
      external initialization, gradient_computation
      external optimization, finalization
      
      call worker_main_body
     +     (initialization, gradient_computation,
     +     optimization, finalization)
      
      end


      subroutine initialization(pp1, lens1, m1)
C     Initialization.
      
      implicit none
      
      include 'worker_error_definition.fh'
      include 'worker_init_definition.fh'
      
C     Output arguments.
      integer pp1(WORKER_MAX_SIZE, WORKER_MAX_DIMENSION)
      integer lens1(WORKER_MAX_DIMENSION)
      FLOAT m1(WORKER_MAX_ZONE_NUMBER)
      
C     Local variables.
      character sbr*14, msg*MAX_ERROR_MSG_LENGTH
      integer s_len, len, dim_loc
      
      s_len = 14
      sbr = 'initialization'
      
      if (.not. ((dim .ge. 1) .or. (dim .le. 4))) then
         call worker_error_unbound_dimension(s_len, sbr, dim)
         return
      endif
      
      call init(dim_loc, size_x, size_y, ier, len, msg)
      if (ier .ne. 0) then
         call worker_error_initialization(s_len, sbr, len, msg)
         return
      endif
      
      if (dim_loc .ne. dim) then
         len = 26 + 1 + 11
         write(msg, 1001) dim_loc
         call worker_error_initialization(s_len, sbr, len, msg)
         return
      endif
      
 1001 format('input image does not have ', i1, ' channel(s)')
      
C     One zone per channel.
      call worker_trivial_init(WORKER_ZERO, pp1, lens1, m1)
      
      return
      end


      subroutine gradient_computation(p, g)
C     Gradient computation.
      
      implicit none
      
      include 'worker_error_definition.fh'
      include 'worker_init_definition.fh'
      
C     Input argument.
      FLOAT p(WORKER_MAX_SIZE, WORKER_MAX_DIMENSION)
      
C     Output argument.
      FLOAT g(WORKER_MAX_SIZE, WORKER_MAX_DIMENSION)
      
C     Local variables.
      character sbr*20, msg*MAX_ERROR_MSG_LENGTH
      integer s_len, len, i, j
      real*8 fine_p(MAX_PIXEL_NUMBER, MAX_CHANNEL_NUMBER)
      real*8 fine_g(MAX_PIXEL_NUMBER, MAX_CHANNEL_NUMBER)
      
      s_len = 20
      sbr = 'gradient_computation'
      
      do j = 1, dim
         do i = 1, size
            fine_p(i, j) = p(i, j)
         enddo
      enddo
      
      call gradient(dim, size, fine_p, fine_g, ier, len, msg)
      if (ier .ne. 0) then
         call worker_error_gradient_computation(s_len, sbr, len, msg)
         return
      endif
      
      do j = 1, dim
         do i = 1, size
            g(i, j) = fine_g(i, j)
         enddo
      enddo
      
      return
      end


      subroutine optimization(pp, lens, m, cost)
C     Optimal coarse parameter computation.
      
      implicit none
      
      include 'worker_error_definition.fh'
      include 'worker_init_definition.fh'
      
C     Input arguments.
      integer pp(WORKER_MAX_SIZE, WORKER_MAX_DIMENSION)
      integer lens(WORKER_MAX_DIMENSION)
      
C     Input/output argument.
      FLOAT m(WORKER_MAX_ZONE_NUMBER)
      
C     Output argument.
      FLOAT cost
      
C     Local variables.
      character sbr*36, msg*MAX_ERROR_MSG_LENGTH
      integer s_len, len, i, j, k
      integer partition(MAX_PIXEL_NUMBER, MAX_CHANNEL_NUMBER)
      integer cp_len(MAX_CHANNEL_NUMBER)
      real*8 coarse_p(MAX_ZONE_NUMBER), jj
      
      s_len = 36
      sbr = 'optimal_coarse_parameter_computation'
      
      do j = 1, dim
         do i = 1, size
            partition(i, j) = pp(i, j)
         enddo
         cp_len(j) = lens(j)
      enddo
      
      do k = 1, zone_number
         coarse_p(k) = m(k)
      enddo
      
      call optim(dim, size, zone_number,
     +     partition, cp_len, coarse_p, jj, ier, len, msg)
      if (ier .ne. 0) then
         call worker_error_optimization(s_len, sbr, len, msg)
         return
      endif
      
      do k = 1, zone_number
         m(k) = coarse_p(k)
      enddo
      
      cost = jj
      
      return
      end


      subroutine finalization(p)
C     Finalization.
      
      implicit none
      
      include 'worker_error_definition.fh'
      include 'worker_init_definition.fh'
      
C     Input argument.
      FLOAT p(WORKER_MAX_SIZE, WORKER_MAX_DIMENSION)
      
C     Local variables.
      character sbr*12, msg*MAX_ERROR_MSG_LENGTH
      integer s_len, len, i, j
      real*8 fine_p(MAX_PIXEL_NUMBER, MAX_CHANNEL_NUMBER)
      
      s_len = 12
      sbr = 'finalization'
      
      do j = 1, dim
         do i = 1, size
            fine_p(i, j) = p(i, j)
         enddo
      enddo
      
      call final(dim, size, fine_p, ier, len, msg)
      if (ier .ne. 0) then
         call worker_error_finalization(s_len, sbr, len, msg)
         return
      endif
      
      return
      end
