CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                     C
C                              Ref-indic                              C
C                                                                     C
C                        Refinement indicators                        C
C                                                                     C
C               EPI Pomdapi, INRIA Paris - Rocquencourt               C
C                                                                     C
C  Copyright (c) 2013-2014 INRIA.                                     C
C  Institut National de Recherche en Informatique et en Automatique.  C
C  All rights reserved. This file is distributed only by permission.  C
C                                                                     C
C  Hend Ben Ameur <hbenameur@yahoo.ca>                                C
C  Francois Clement <Francois.Clement@inria.fr>                       C
C  Pierre Weis <Pierre.Weis@inria.fr>                                 C
C                                                                     C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

C     $Id: worker_error.fpp,v 1.10 2014/06/18 13:33:02 fclement Exp $

#include "worker_constants.h"

      subroutine worker_error_initialization(s_len, sbr, len, msg)
      
      implicit none
      
C     Input arguments.
      character sbr*(*), msg*(*)
      integer s_len, len
      
      call pio_user_error_task_computation
     +     (s_len, sbr, 4, 'init', len, msg)
      
      return
      end


      subroutine worker_error_gradient_computation(s_len, sbr, len, msg)
      
      implicit none
      
C     Input arguments.
      character sbr*(*), msg*(*)
      integer s_len, len
      
      call pio_user_error_task_computation
     +     (s_len, sbr, 4, 'grad', len, msg)
      
      return
      end


      subroutine worker_error_optimization(s_len, sbr, len, msg)
      
      implicit none
      
C     Input arguments.
      character sbr*(*), msg*(*)
      integer s_len, len
      
      call pio_user_error_task_computation
     +     (s_len, sbr, 5, 'optim', len, msg)
      
      return
      end


      subroutine worker_error_finalization(s_len, sbr, len, msg)
      
      implicit none
      
C     Input arguments.
      character sbr*(*), msg*(*)
      integer s_len, len
      
      call pio_user_error_task_computation
     +     (s_len, sbr, 5, 'final', len, msg)
      
      return
      end


      subroutine worker_error_wrong_dimension(s_len, sbr, good, bad)
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer s_len, good, bad
      
      call pio_user_error_wrong_length
     +     (s_len, sbr, 9, 'dimension', good, bad)
      
      return
      end


      subroutine worker_error_wrong_size(s_len, sbr, good, bad)
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer s_len, good, bad
      
      call pio_user_error_wrong_length
     +     (s_len, sbr, 4, 'size', good, bad)
      
      return
      end


      subroutine worker_error_wrong_zone_number(s_len, sbr, good, bad)
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer s_len, good, bad
      
      call pio_user_error_wrong_length
     +     (s_len, sbr, 11, 'zone number', good, bad)
      
      return
      end


      subroutine worker_error_too_large_dimension(s_len, sbr, bad)
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer s_len, bad
      
      call pio_user_error_too_large_length
     +     (s_len, sbr, 9, 'dimension', WORKER_MAX_DIMENSION, bad)
      
      return
      end


      subroutine worker_error_too_large_size(s_len, sbr, bad)
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer s_len, bad
      
      call pio_user_error_too_large_length
     +     (s_len, sbr, 4, 'size', WORKER_MAX_SIZE, bad)
      
      return
      end


      subroutine worker_error_too_large_zone_number(s_len, sbr, bad)
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer s_len, bad
      
      call pio_user_error_too_large_length
     +     (s_len, sbr, 11, 'zone number', WORKER_MAX_ZONE_NUMBER, bad)
      
      return
      end


      subroutine worker_error_too_small_dimension(s_len, sbr, bad)
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer s_len, bad
      
      call pio_user_error_too_small_length
     +     (s_len, sbr, 9, 'dimension', 1, bad)
      
      return
      end


      subroutine worker_error_too_small_size(s_len, sbr, bad)
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer s_len, bad
      
      call pio_user_error_too_small_length
     +     (s_len, sbr, 4, 'size', 1, bad)
      
      return
      end


      subroutine worker_error_too_small_zone_number(s_len, sbr, bad)
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer s_len, bad
      
      call pio_user_error_too_small_length
     +     (s_len, sbr, 11, 'zone number', 0, bad)
      
      return
      end


      subroutine error_not_yet_implemented(s_len, sbr, len, msg)
      
      implicit none
      
      include 'pio_user_error_definition.fh'
      
C     Input arguments.
      character sbr*(*), msg*(*)
      integer s_len, len
      
      ier = WORKER_NOT_YET_IMPLEMENTED_ERROR
      
      em_len = 15 + s_len + 1 + 1 + len + 21
      em_len = min(em_len, PIO_USER_MAX_ERROR_MESSAGE_LENGTH)
      
      write(error_msg, 1001) sbr(1:s_len), msg(1:len)
      
 1001 format('fatal error in ', a, ':',
     +     ' ', a, ' not yet implemented.')
      
      return
      end


      subroutine worker_error_unbound_dimension(s_len, sbr, bad)
      
      implicit none
      
      include 'pio_user_error_definition.fh'
      
C     Input arguments.
      character sbr*(*)
      integer s_len, bad
      
      ier = WORKER_UNBOUND_DIMENSION_ERROR
      
      em_len = 15 + s_len + 1 + 19 + WORKER_MAX_DIMENSION_LEN + 1
      em_len = min(em_len, PIO_USER_MAX_ERROR_MESSAGE_LENGTH)
      
      write(error_msg, 1001) sbr(1:s_len), bad
      
 1001 format('fatal error in ', a, ':',
     +     ' unbound dimension ', WORKER_MAX_DIMENSION_FMT, '.')
      
      return
      end


      subroutine worker_error_open_failure(s_len, sbr, len, fname)
      
      implicit none
      
      include 'pio_user_error_definition.fh'
      
C     Input arguments.
      character sbr*(*), fname*(*)
      integer s_len, len
      
      call pio_user_error_open_failure(s_len, sbr, len, fname)
      
      return
      end


      subroutine worker_error_wrong_length
     +     (s_len, sbr, ln_len, length_name, good, bad)
      
      implicit none
      
      include 'pio_user_error_definition.fh'
      
C     Input arguments.
      character sbr*(*), length_name*(*)
      integer s_len, ln_len, good, bad
      
      call pio_user_error_wrong_length
     +     (s_len, sbr, ln_len, length_name, good, bad)
      
      return
      end


      subroutine worker_error_too_large_length
     +     (s_len, sbr, ln_len, length_name, good, bad)
      
      implicit none
      
      include 'pio_user_error_definition.fh'
      
C     Input arguments.
      character sbr*(*), length_name*(*)
      integer s_len, ln_len, good, bad
      
      call pio_user_error_too_large_length
     +     (s_len, sbr, ln_len, length_name, good, bad)
      
      return
      end


      subroutine worker_error_too_small_length
     +     (s_len, sbr, ln_len, length_name, good, bad)
      
      implicit none
      
      include 'pio_user_error_definition.fh'
      
C     Input arguments.
      character sbr*(*), length_name*(*)
      integer s_len, ln_len, good, bad
      
      call pio_user_error_too_small_length
     +     (s_len, sbr, ln_len, length_name, good, bad)
      
      return
      end


      logical function worker_get_debug_mode()
      
      implicit none
      
C     External function.
      logical pio_user_get_debug_mode
      
      worker_get_debug_mode = pio_user_get_debug_mode()
      
      return
      end


      logical function worker_get_report_mode()
      
      implicit none
      
C     External function.
      logical pio_user_get_report_mode
      
      worker_get_report_mode = pio_user_get_report_mode()
      
      return
      end
